# coding=utf-8
# Constants for the various options go here...


class YAxisFormat:
    NoFormat = 'none'
    Short = 'short'
    Bytes = 'bytes'
    Bits = 'bits'
    BitsPerSecond = 'bps'
    Seconds = 's'
    Milliseconds = 'ms'
    Microseconds = 'µs'
    Nanoseconds = 'ns'
    Percent = 'percent'

class YAxisMinimum:
    Zero = 0
    Auto = None

class StackStyle:
    Stacked = True
    Unstacked = False


class FillStyle:
    Filled = 10
    Unfilled = 0


class Metric:
    def __init__(self, target):
        self.target = target

    def build(self):
        return {
            "target": self.target
        }


class Panel:
    def __init__(self, title, y_axis_format=YAxisFormat.NoFormat, filled=FillStyle.Unfilled, stacked=StackStyle.Unstacked, minimum=None):
        self.y_axis_format = y_axis_format
        self.title = title
        self.metrics = []
        self.filled = filled
        self.stacked = stacked
        self.minimum = minimum

    def with_metric(self, metric):
        self.metrics.append(metric.build())
        return self

    def with_metrics(self, metrics):
        self.metrics += metrics
        return self

    def build(self, panel_id, span=12):
        return {
            "title": self.title,
            "error": False,
            "span": span,
            "editable": True,
            "type": "graph",
            "id": panel_id,
            "datasource": None,
            "renderer": "flot",
            "x-axis": True,
            "y-axis": True,
            "y_formats": [
                self.y_axis_format,
                "none"
            ],
            "grid": {
                "leftMax": None,
                "rightMax": None,
                "leftMin": self.minimum,
                "rightMin": None,
                "threshold1": None,
                "threshold2": None,
                "threshold1Color": "rgba(216, 200, 27, 0.27)",
                "threshold2Color": "rgba(234, 112, 112, 0.22)"
            },
            "lines": True,
            "fill": self.filled,
            "linewidth": 1,
            "points": False,
            "pointradius": 5,
            "bars": False,
            "stack": self.stacked,
            "percentage": False,
            "legend": {
                "show": True,
                "values": False,
                "min": False,
                "max": False,
                "current": False,
                "total": False,
                "avg": False
            },
            "NonePointMode": "connected",
            "steppedLine": False,
            "tooltip": {
                "value_type": "cumulative",
                "shared": False
            },
            "targets": self.metrics,
            "aliasColors": {},
            "seriesOverrides": [],
            "links": []
        }


class Row:
    def __init__(self):
        self.panels = []

    def with_panel(self, panel):
        self.panels.append(panel)
        return self

    def with_panels(self, panels):
        self.panels += panels
        return self

    def build(self, row_id):
        def to_panel(panel_builder):
            return panel_builder.build(self.panels.index(panel_builder), 12 / len(self.panels))

        return {
            "title": "Row %d" % row_id,
            "height": "250px",
            "editable": True,
            "collapse": False,
            "panels": map(to_panel, self.panels)
        }


class Dashboard:
    def __init__(self, title):
        self.title = title
        self.rows = []

    def with_row(self, row):
        self.rows.append(row.build(len(self.rows)))
        return self

    def with_rows(self, rows):
        self.rows += rows
        return self

    def build(self):
        return {
            "title": self.title,
            "originalTitle": self.title,
            "tags": [],
            "style": "dark",
            "timezone": "browser",
            "editable": True,
            "hideControls": False,
            "sharedCrosshair": False,
            "rows": self.rows,
            "nav": [
                {
                    "type": "timepicker",
                    "enable": True,
                    "status": "Stable",
                    "time_options": [
                        "5m",
                        "15m",
                        "1h",
                        "6h",
                        "12h",
                        "24h",
                        "2d",
                        "7d",
                        "30d"
                    ],
                    "refresh_intervals": [
                        "5s",
                        "10s",
                        "30s",
                        "1m",
                        "5m",
                        "15m",
                        "30m",
                        "1h",
                        "2h",
                        "1d"
                    ],
                    "now": True,
                    "collapse": False,
                    "notice": False
                }
            ],
            "time": {
                "from": "now-15m",
                "to": "now"
            },
            "templating": {
                "list": []
            },
            "annotations": {
                "list": [],
                "enable": False
            },
            "refresh": "10s",
            "version": 6,
            "hideAllLegends": False
        }



class Filter:
    def __init__(self):
        self.filters = {
            "0": {
                "type": "time",
                "field": "@timestamp",
                "from": "now-1h",
                "to": "now",
                "mandate": "must",
                "active": True,
                "alias": "",
                "id": 0
            }
        }

    def with_field(self, field, value):
        next_index = len(self.filters)
        self.filters[str(next_index)] = {
            "type": "field",
            "field": field,
            "query": "\"" + value + "\"",
            "mandate": "must",
            "active": True,
            "alias": "",
            "id": next_index
        }
        return self

    def build(self):
        return {
            "list": self.filters,
            "ids": range(0, len(self.filters))
        }


class Dashboard:
    def __init__(self, title):
        self.title = title
        self.filter = Filter()
        self.fields = []

    def with_filter(self, new_filter):
        self.filter = new_filter
        return self

    def with_fields(self, fields):
        self.fields = fields
        return self

    def build(self):
        return {
            "title": self.title,
            "services": {
                "query": {
                    "list": {
                        "0": {
                            "query": "",
                            "alias": "",
                            "color": "#7EB26D",
                            "id": 0,
                            "pin": False,
                            "type": "lucene",
                            "enable": True
                        }
                    },
                    "ids": [0]
                },
                "filter": self.filter.build()
            },
            "rows": [
                {
                    "title": "Graph",
                    "height": "350px",
                    "editable": True,
                    "collapse": False,
                    "collapsable": True,
                    "panels": [
                        {
                            "span": 12,
                            "editable": True,
                            "group": ["default"],
                            "type": "histogram",
                            "mode": "count",
                            "time_field": "@timestamp",
                            "value_field": None,
                            "auto_int": True,
                            "resolution": 100,
                            "interval": "30s",
                            "fill": 3,
                            "linewidth": 3,
                            "timezone": "browser",
                            "spyable": True,
                            "zoomlinks": True,
                            "bars": True,
                            "stack": True,
                            "points": False,
                            "lines": False,
                            "legend": True,
                            "x-axis": True,
                            "y-axis": True,
                            "percentage": False,
                            "interactive": True,
                            "queries": {
                                "mode": "all",
                                "ids": [0]
                            },
                            "title": "Events over time",
                            "intervals": [
                                "auto",
                                "1s",
                                "1m",
                                "5m",
                                "10m",
                                "30m",
                                "1h",
                                "3h",
                                "12h",
                                "1d",
                                "1w",
                                "1M",
                                "1y"
                            ],
                            "options": True,
                            "tooltip": {
                                "value_type": "cumulative",
                                "query_as_alias": True
                            },
                            "scale": 1,
                            "y_format": "none",
                            "grid": {
                                "max": None,
                                "min": 0
                            },
                            "annotate": {
                                "enable": False,
                                "query": "*",
                                "size": 20,
                                "field": "_type",
                                "sort": [
                                    "_score",
                                    "desc"
                                ]
                            },
                            "pointradius": 5,
                            "show_query": True,
                            "legend_counts": True,
                            "zerofill": True,
                            "derivative": False
                        }
                    ],
                    "notice": False
                },
                {
                    "notice": False,
                    "panels": [
                        {
                            "status": "Stable",
                            "header": True,
                            "trimFactor": 300,
                            "spyable": True,
                            "field_list": True,
                            "size": 100,
                            "all_fields": False,
                            "style": {
                                "font-size": "9pt"
                            },
                            "span": 12,
                            "title": "All events",
                            "normTimes": True,
                            "type": "table",
                            "sort": [
                                "@timestamp",
                                "desc"
                            ],
                            "error": False,
                            "editable": True,
                            "offset": 0,
                            "group": [
                                "default"
                            ],
                            "overflow": "min-height",
                            "pages": 5,
                            "localTime": True,
                            "sortable": True,
                            "fields": self.fields,
                            "paging": True,
                            "queries": {
                                "mode": "all",
                                "ids": [
                                    0
                                ]
                            },
                            "timeField": "@timestamp",
                            "highlight": []
                        }
                    ],
                    "collapse": False,
                    "title": "Events",
                    "editable": True,
                    "height": "350px",
                    "collapsable": True
                }
            ],
            "editable": True,
            "failover": False,
            "index": {
                "interval": "day",
                "pattern": "[logstash-]YYYY.MM.DD",
                "default": "NO_TIME_FILTER_OR_INDEX_PATTERN_NOT_MATCHED",
                "warm_fields": True
            },
            "style": "dark",
            "panel_hints": True,
            "pulldowns": [
                {
                    "type": "query",
                    "collapse": False,
                    "notice": False,
                    "query": "*",
                    "pinned": True,
                    "history": [
                    ],
                    "remember": 10,
                    "enable": True
                },
                {
                    "type": "filtering",
                    "collapse": False,
                    "notice": True,
                    "enable": True
                }
            ],
            "nav": [
                {
                    "type": "timepicker",
                    "collapse": False,
                    "notice": False,
                    "status": "Stable",
                    "time_options": [
                        "5m",
                        "15m",
                        "1h",
                        "6h",
                        "12h",
                        "24h",
                        "2d",
                        "7d",
                        "30d"
                    ],
                    "refresh_intervals": [
                        "5s",
                        "10s",
                        "30s",
                        "1m",
                        "5m",
                        "15m",
                        "30m",
                        "1h",
                        "2h",
                        "1d"
                    ],
                    "timefield": "@timestamp",
                    "now": True,
                    "filter_id": 0,
                    "enable": True
                }
            ],
            "loader": {
                "save_gist": False,
                "save_elasticsearch": True,
                "save_local": True,
                "save_default": True,
                "save_temp": True,
                "save_temp_ttl_enable": True,
                "save_temp_ttl": "30d",
                "load_gist": True,
                "load_elasticsearch": True,
                "load_elasticsearch_size": 20,
                "load_local": True,
                "hide": False
            },
            "refresh": "30s"
        }




from setuptools import setup, find_packages
import os

setup(
    name='balderdash',
    version='1.3.1',
    description='Builders for monitoring dashboards',
    url='http://github.com/daviddenton/balderdash/',
    license='Apache 2.0',
    author='daviddenton',
    author_email='denton.david+pypy@gmail.com',
    packages=find_packages(exclude=['tests*']),
    test_suite="tests",
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)